@@ -36,7 +36,7 @@
 import pytest
 from fparser.two.parser import ParserFactory
 from fparser.common.readfortran import FortranStringReader
-from fparser.two.utils import FortranSyntaxError
+from fparser.two.utils import FortranSyntaxError, StmtBase
 from fparser.two.symbol_table import SYMBOL_TABLES
 from fparser.two import Fortran2003, Fortran2008
 
@@ -84,3 +84,112 @@ def test_parserfactory_std():
     with pytest.raises(ValueError) as excinfo:
         parser = ParserFactory().create(std="invalid")
         assert "is an invalid standard" in str(excinfo.value)
+
+
+def _cmp_tree_types_rec(
+    node1: Fortran2003.Program, node2: Fortran2003.Program, depth: int = 0
+):
+    """Helper function to recursively check for deepcopied programs
+
+    :param node1: First AST tree to check
+    :type node1: Fortran2003.Program
+    :param node2: Second AST tree to check
+    :type node2: Fortran2003.Program
+    :param depth: Depth useful later on for debugging reasons,
+        defaults to 0
+    :type depth: int, optional
+    """
+
+    # Make sure that both trees are the same
+    assert type(node1) is type(
+        node2
+    ), f"Nodes have different types: '{type(node1)}' and '{type(node2)}"
+
+    if node1 is None:
+        # Just return for None objects
+        return
+
+    if type(node1) is str:
+        # WARNING: Different string objects with the same can have the same id.
+        # Therefore, we can't compare with 'is' or with 'id(.) == id(.)'.
+        # We can just compare the both strings have the same content.
+        # See https://stackoverflow.com/questions/20753364/why-does-creating-multiple-objects-without-naming-them-result-in-them-having-the
+        assert node1 == node2
+        return
+
+    else:
+        # Make sure that we're working on a copy rather than the same object
+        assert node1 is not node2, "Nodes refer to the same object"
+
+    # Continue recursive traversal of ast
+    for child1, child2 in zip(node1.children, node2.children):
+        _cmp_tree_types_rec(child1, child2, depth + 1)
+
+
+_f90_source_test = """
+module andy
+implicit none
+
+   real :: apple = 1.0
+   real, parameter :: pi = 3.14
+
+contains
+   subroutine sergi()
+      print *, "Pi = ", pi
+      print *,  "apple = ", apple
+   end subroutine
+
+end module andy
+
+
+program awesome
+    use andy
+    implicit none
+
+   real :: x
+   integer :: i
+
+   x = 2.2
+   i = 7
+
+   call sergi()
+
+   print *, "apple pie: ", apple, pi
+   print *, "i: ", i
+
+end program awesome
+
+"""
+
+
+def test_deepcopy():
+    """
+    Test that we can deepcopy a parsed fparser tree.
+    """
+
+    parser = ParserFactory().create(std="f2008")
+    reader = FortranStringReader(_f90_source_test)
+    ast = parser(reader)
+
+    import copy
+
+    new_ast = copy.deepcopy(ast)
+
+    _cmp_tree_types_rec(new_ast, ast)
+
+
+def test_pickle():
+    """
+    Test that we can pickle and unpickle a parsed fparser tree.
+    """
+
+    parser = ParserFactory().create(std="f2008")
+    reader = FortranStringReader(_f90_source_test)
+    ast = parser(reader)
+
+    import pickle
+
+    s = pickle.dumps(ast)
+    new_ast = pickle.loads(s)
+
+    _cmp_tree_types_rec(new_ast, ast)
